=== modified file 'lib/canonical/launchpad/scripts/productreleasefinder/filter.py'
--- lib/canonical/launchpad/scripts/productreleasefinder/filter.py	2006-09-01 06:18:33 +0000
+++ lib/canonical/launchpad/scripts/productreleasefinder/filter.py	2006-09-15 09:20:12 +0000
@@ -37,16 +37,27 @@
         and returns the 'key' attribute of the first pattern that
         matches.
         """
-        self.log.info("Checking %s", url)
+        self.log.debug("Checking %s", url)
         for pattern in self.filters:
             if pattern.match(url):
-                self.log.info("Matches %s glob (%s)",
-                              pattern.key, pattern.glob)
+                self.log.info("%s matches %s glob (%s)",
+                              url, pattern.key, pattern.glob)
                 return pattern.key
         else:
-            self.log.info("No matches")
+            self.log.debug("No matches")
             return None
 
+    def isPossibleParent(self, url):
+        """Check if any filters could match children of a URL."""
+        self.log.debug("Checking if %s is a possible parent", url)
+        for pattern in self.filters:
+            if pattern.containedBy(url):
+                self.log.info("%s could contain matches for %s glob (%s)",
+                              url, pattern.key, pattern.glob)
+                return True
+        else:
+            return False
+
 
 class FilterPattern:
     """A filter pattern.
@@ -62,12 +73,33 @@
 
         if not self.base_url.endswith('/'):
             self.base_url += '/'
-        regexp = fnmatch.translate(self.base_url + self.glob)
-        # Use the same hack as distutils does so that "*" and "?" in
-        # globs do not match slashes.
-        regexp = re.sub(r'(^|[^\\])\.', r'\1[^/]', regexp)
-        self.pattern = re.compile(regexp)
+        parts = (self.base_url + self.glob).split('/')
+        self.patterns = [re.compile(fnmatch.translate(part)) for part in parts]
 
     def match(self, url):
         """Returns true if this filter pattern matches the URL."""
-        return bool(self.pattern.match(url))
+        parts = url.split('/')
+        # If the length of list of slash separated parts of the URL
+        # differs from the number of patterns, then they can't match.
+        if len(parts) != len(self.patterns):
+            return False
+        for (part, pattern) in zip(parts, self.patterns):
+            if not pattern.match(part):
+                return False
+        # Everything matches ...
+        return True
+
+    def containedBy(self, url):
+        """Returns true if this pattern could match children of the URL."""
+        url = url.rstrip('/')
+        parts = url.split('/')
+        # If the URL contains greater than or equal the number of
+        # parts as the number of patterns we have, then it couldn't
+        # contain any children that match this pattern.
+        if len(parts) >= len(self.patterns):
+            return False
+        for (part, pattern) in zip(parts, self.patterns):
+            if not pattern.match(part):
+                return False
+        # Everything else matches ...
+        return True

=== modified file 'lib/canonical/launchpad/scripts/productreleasefinder/finder.py'
--- lib/canonical/launchpad/scripts/productreleasefinder/finder.py	2006-09-07 06:00:39 +0000
+++ lib/canonical/launchpad/scripts/productreleasefinder/finder.py	2006-09-15 09:20:12 +0000
@@ -84,6 +84,8 @@
             if series_name is not None:
                 try:
                     self.handleRelease(product_name, series_name, url)
+                except (KeyboardInterrupt, SystemExit):
+                    raise
                 except:
                     self.log.exception("Could not successfully process "
                                        "URL %s for %s/%s",
@@ -114,7 +116,7 @@
     def addReleaseTarball(self, product_name, series_name, release_name,
                           filename, size, file, content_type):
         """Create a ProductRelease (if needed), and attach tarball"""
-        # get the series
+        # Get the series.
         self.ztm.begin()
         try:
             product = getUtility(IProductSet).getByName(product_name)
@@ -128,7 +130,7 @@
                 self.log.info("Created new release %s for %s/%s",
                               release_name, product_name, series_name)
 
-            # if we already have a code tarball, stop here
+            # If we already have a code tarball, stop here.
             for fileinfo in release.files:
                 if fileinfo.filetype == UpstreamFileType.CODETARBALL:
                     self.log.debug("%s/%s/%s already has a code tarball",

=== modified file 'lib/canonical/launchpad/scripts/productreleasefinder/hose.py'
--- lib/canonical/launchpad/scripts/productreleasefinder/hose.py	2006-09-06 11:30:13 +0000
+++ lib/canonical/launchpad/scripts/productreleasefinder/hose.py	2006-09-15 09:20:12 +0000
@@ -62,5 +62,15 @@
                     url = combine_url(base_url, dirpath, filename)
                     key = self.filter.check(url)
                     yield (key, url)
+                # To affect which directories the walker descends
+                # into, we must update the dirnames list in place.
+                i = 0
+                while i < len(dirnames):
+                    url = combine_url(base_url, dirpath, dirnames[i])
+                    if self.filter.isPossibleParent(url):
+                        i += 1
+                    else:
+                        self.log.info('Skipping %s', url)
+                        del dirnames[i]
 
     __iter__ = run

=== modified file 'lib/canonical/launchpad/scripts/productreleasefinder/tests/test_filter.py'
--- lib/canonical/launchpad/scripts/productreleasefinder/tests/test_filter.py	2006-09-01 06:18:33 +0000
+++ lib/canonical/launchpad/scripts/productreleasefinder/tests/test_filter.py	2006-09-13 08:03:56 +0000
@@ -47,45 +47,66 @@
         f = Filter()
         self.assertEquals(f.check("file:///subdir/file"), None)
 
+    def makeFilter(self, key, base_url, glob):
+        from canonical.launchpad.scripts.productreleasefinder.filter import (
+            Filter, FilterPattern)
+        pattern = FilterPattern(key, base_url, glob)
+        return Filter([pattern])
+
     def testNotMatching(self):
         """Filter.check returns None if doesn't match a filter."""
-        from canonical.launchpad.scripts.productreleasefinder.filter import (
-            Filter, FilterPattern)
-        pattern = FilterPattern("foo", "file:///subdir", "w*")
-        f = Filter([pattern])
+        f = self.makeFilter("foo", "file:///subdir", "w*")
         self.assertEquals(f.check("file:///subdir/file"), None)
 
     def testNoMatchingSlashes(self):
         """Filter.check that the glob does not match slashes."""
-        from canonical.launchpad.scripts.productreleasefinder.filter import (
-            Filter, FilterPattern)
-        pattern = FilterPattern("foo", "file:///", "*d*")
-        f = Filter([pattern])
+        f = self.makeFilter("foo", "file:///", "*d*")
         self.assertEquals(f.check("file:///subdir/file"), None)
 
     def testReturnsMatching(self):
         """Filter.check returns the matching keyword."""
-        from canonical.launchpad.scripts.productreleasefinder.filter import (
-            Filter, FilterPattern)
-        pattern = FilterPattern("foo", "file:///subdir", "f*e")
-        f = Filter([pattern])
+        f = self.makeFilter("foo", "file:///subdir", "f*e")
         self.assertEquals(f.check("file:///subdir/file"), "foo")
 
     def testGlobSubdir(self):
         # Filter.glob can contain slashes to match subdirs
-        from canonical.launchpad.scripts.productreleasefinder.filter import (
-            Filter, FilterPattern)
-        pattern = FilterPattern("foo", "file:///", "sub*/f*e")
-        f = Filter([pattern])
+        f = self.makeFilter("foo", "file:///", "sub*/f*e")
         self.assertEquals(f.check("file:///subdir/file"), "foo")
 
     def testReturnsNonMatchingBase(self):
         """Filter.check returns None if the base does not match."""
+        f = self.makeFilter("foo", "http:", "f*e")
+        self.assertEquals(f.check("file:///subdir/file"), None)
+
+
+class Filter_IsPossibleParentUrl(unittest.TestCase):
+
+    def makeFilter(self, key, base_url, glob):
         from canonical.launchpad.scripts.productreleasefinder.filter import (
             Filter, FilterPattern)
-        pattern = FilterPattern("foo", "http:", "f*e")
-        f = Filter([pattern])
-        self.assertEquals(f.check("file:///subdir/file"), None)
+        pattern = FilterPattern(key, base_url, glob)
+        return Filter([pattern])
+
+    def testNotContainedByMatch(self):
+        # if the URL matches the pattern, then it can't contain matches.
+        f = self.makeFilter("foo", "file:///subdir", "foo-1.*.tar.gz")
+        self.assertFalse(f.isPossibleParent("file:///subdir/foo-1.42.tar.gz"))
+
+    def testContainedByParent(self):
+        # parent directories of the match can contain the match
+        f = self.makeFilter("foo", "file:///subdir", "foo/bar")
+        self.assertTrue(f.isPossibleParent("file:///subdir/foo/"))
+        self.assertTrue(f.isPossibleParent("file:///subdir/foo"))
+        self.assertTrue(f.isPossibleParent("file:///subdir"))
+        self.assertTrue(f.isPossibleParent("file:///"))
+
+    def testContainedByGlobbedParent(self):
+        # test that glob matched parents can contain matches
+        f = self.makeFilter("foo", "file:///subdir", "1.*/foo-1.*.tar.gz")
+        self.assertTrue(f.isPossibleParent("file:///subdir/1.0/"))
+        self.assertTrue(f.isPossibleParent("file:///subdir/1.42"))
+        self.assertTrue(f.isPossibleParent("file:///subdir/1.abc/"))
+        self.assertFalse(f.isPossibleParent("file:///subdir/2.0"))
 
 
 def test_suite():

=== modified file 'lib/canonical/launchpad/scripts/productreleasefinder/tests/test_hose.py'
--- lib/canonical/launchpad/scripts/productreleasefinder/tests/test_hose.py	2006-09-01 06:18:33 +0000
+++ lib/canonical/launchpad/scripts/productreleasefinder/tests/test_hose.py	2006-09-15 09:20:12 +0000
@@ -1,6 +1,11 @@
 """Tests for canonical.launchpad.scripts.productreleasefinder.hose."""
 
+import os
+import shutil
+import tempfile
 import unittest
+
+from canonical.testing import reset_logging
 from hct.scaffold import Scaffold
 
 
@@ -100,5 +105,69 @@
                           ["http://localhost/", "file:///usr/"])
 
 
+class Hose_LimitWalk(unittest.TestCase):
+
+    def setUp(self):
+        self.release_root = tempfile.mkdtemp()
+        self.release_url = 'file://' + self.release_root
+
+    def tearDown(self):
+        shutil.rmtree(self.release_root, ignore_errors=True)
+        reset_logging()
+
+    def testHoseLimitsWalk(self):
+        # Test that the hose limits the directory walk to places that
+        # could contain a match.
+        
+        # Set up the releases tree:
+        for directory in ['bar',
+                          'foo',
+                          'foo/1.0',
+                          'foo/1.0/source',
+                          'foo/1.0/x64',
+                          'foo/1.5',
+                          'foo/1.5/source',
+                          'foo/2.0',
+                          'foo/2.0/source']:
+            os.mkdir(os.path.join(self.release_root, directory))
+        for releasefile in ['foo/1.0/foo-1.0.tar.gz',
+                            'foo/1.0/source/foo-1.0.tar.gz',
+                            'foo/1.0/source/foo-2.0.tar.gz',
+                            'foo/1.0/x64/foo-1.0.tar.gz',
+                            'foo/1.5/source/foo-1.5.tar.gz',
+                            'foo/2.0/source/foo-2.0.tar.gz']:
+            fp = open(os.path.join(self.release_root, releasefile), 'wb')
+            fp.write('data')
+            fp.close()
+
+        # Run the hose over the test data
+        from canonical.launchpad.scripts.productreleasefinder.hose import Hose
+        from canonical.launchpad.scripts.productreleasefinder.filter import (
+            FilterPattern)
+        pattern = FilterPattern("key", self.release_url,
+                                "foo/1.*/source/foo-1.*.tar.gz")
+        hose = Hose([pattern])
+
+        prefix_len = len(self.release_url)
+        matched = []
+        unmatched = []
+        for key, url in hose:
+            if key is None:
+                unmatched.append(url[prefix_len:])
+            else:
+                matched.append(url[prefix_len:])
+
+        # Make sure that the correct releases got found.
+        self.assertEqual(sorted(matched),
+                         ['/foo/1.0/source/foo-1.0.tar.gz',
+                          '/foo/1.5/source/foo-1.5.tar.gz'])
+
+        # The only unmatched files that get checked exist in
+        # directories that are parents of potential matches.
+        self.assertEqual(sorted(unmatched),
+                         ['/foo/1.0/foo-1.0.tar.gz',
+                          '/foo/1.0/source/foo-2.0.tar.gz'])
+
+
 def test_suite():
     return unittest.TestLoader().loadTestsFromName(__name__)

=== modified file 'lib/canonical/launchpad/scripts/productreleasefinder/tests/test_walker.py'
--- lib/canonical/launchpad/scripts/productreleasefinder/tests/test_walker.py	2006-09-06 11:30:13 +0000
+++ lib/canonical/launchpad/scripts/productreleasefinder/tests/test_walker.py	2006-09-15 09:20:12 +0000
@@ -251,7 +251,7 @@
         return TestHTTPWalker(listing_url, logging.getLogger())
 
     def testApacheListing(self):
-        # Test that list() handles a standard Apache dir listing
+        # Test that list() handles a standard Apache dir listing.
         content = '''
 <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 3.2 Final//EN">
 <html>
@@ -281,7 +281,7 @@
                                      'gnome-gpg-0.5.0.tar.gz'])
 
     def testSquidFtpListing(self):
-        # Test that a Squid FTP listing can be parsed
+        # Test that a Squid FTP listing can be parsed.
         content = '''
 <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
 <!-- HTML listing generated by Squid 2.5.STABLE12 -->
@@ -315,7 +315,7 @@
                                      'gnome-gpg-0.5.0.tar.gz'])
 
     def testNonAsciiListing(self):
-        # Test that list() handles non-ASCII output
+        # Test that list() handles non-ASCII output.
         content = '''
         <html>
           <head>
@@ -345,7 +345,7 @@
         self.assertEqual(filenames, ['file1', 'file2', 'file3', 'file99'])
 
     def testGarbageListing(self):
-        # make sure that garbage doesn't trip up the dir lister
+        # Make sure that garbage doesn't trip up the dir lister.
         content = '\x01\x02\x03\x00\xff\xf2\xablkjsdflkjsfkljfds'
         walker = self.setUpWalker('http://example.com/foo/', content)
         dirnames, filenames = walker.list('/foo/')
@@ -353,6 +353,28 @@
         self.assertEqual(filenames, [])
 
 
+class HTTPWalker_IsDirectory(unittest.TestCase):
+    
+    def tearDown(self):
+        reset_logging()
+
+    def testFtpIsDirectory(self):
+        # Test that no requests are made by isDirectory() when walking
+        # FTP sites.
+        from canonical.launchpad.scripts.productreleasefinder.walker import (
+            HTTPWalker)
+        test = self
+        class TestHTTPWalker(HTTPWalker):
+            def request(self, method, path):
+                test.fail('%s was requested with method %s' % (path, method))
+
+        logging.basicConfig(level=logging.CRITICAL)
+        walker = TestHTTPWalker('ftp://ftp.gnome.org/', logging.getLogger())
+
+        self.assertEqual(walker.isDirectory('/foo/'), True)
+        self.assertEqual(walker.isDirectory('/foo'), False)
+
+
 class Walker_CombineUrl(unittest.TestCase):
     def testConstructsUrl(self):
         """combine_url constructs the URL correctly."""
@@ -360,6 +382,8 @@
             combine_url)
         self.assertEquals(combine_url("file:///base", "/subdir/", "file"),
                           "file:///subdir/file")
+        self.assertEquals(combine_url("file:///base", "/subdir", "file"),
+                          "file:///subdir/file")
 
 
 def test_suite():

=== modified file 'lib/canonical/launchpad/scripts/productreleasefinder/walker.py'
--- lib/canonical/launchpad/scripts/productreleasefinder/walker.py	2006-09-06 12:00:43 +0000
+++ lib/canonical/launchpad/scripts/productreleasefinder/walker.py	2006-09-15 09:35:15 +0000
@@ -22,6 +22,7 @@
 
 from hct.util import log
 from hct.util.path import as_dir, subdir, under_only
+from canonical.launchpad.webapp.url import urlappend
 
 
 class WalkerError(Exception): pass
@@ -260,7 +261,7 @@
 
         Returns the HTTPResponse object.
         """
-        # we build a custom opener, because we don't want redirects to be
+        # We build a custom opener, because we don't want redirects to be
         # followed.
         if self._opener is None:
             self._opener = urllib2.OpenerDirector()
@@ -287,6 +288,12 @@
         if path.endswith("/"):
             return True
 
+        # If the URI scheme is FTP, then the URI comes from a Squid
+        # FTP listing page, which includes the trailing slash on all
+        # URIs that need it.
+        if self.scheme == 'ftp':
+            return False
+
         self.log.debug("Checking if %s is a directory" % path)
         try:
             response = self.request("HEAD", path)
@@ -295,10 +302,10 @@
             if exc.code != 301:
                 return False
         except (IOError, socket.error), exc:
-            # raise HTTPWalkerError for other IO or socket errors
+            # Raise HTTPWalkerError for other IO or socket errors.
             raise HTTPWalkerError(str(exc))
 
-        # we have a 301 redirect error from here on.
+        # We have a 301 redirect error from here on.
         url = exc.hdrs.getheader("location")
         (scheme, netloc, redirect_path, query, fragment) \
                  = urlsplit(url, self.scheme, self.FRAGMENTS)
@@ -365,7 +372,7 @@
     """Return a walker for the URL given."""
     (scheme, netloc, path, query, fragment) = urlsplit(url, "file")
     if scheme in ["ftp"]:
-        # if ftp_proxy is set, use the HTTPWalker class since we are
+        # If ftp_proxy is set, use the HTTPWalker class since we are
         # talking to an HTTP proxy.
         if 'ftp_proxy' in os.environ:
             return HTTPWalker(url, log_parent)
@@ -381,4 +388,5 @@
 def combine_url(base, subdir, filename):
     """Combine a URL from the three parts returned by walk()."""
     subdir_url = urljoin(base, subdir)
-    return urljoin(subdir_url, filename)
+    # The "filename" component must be appended to the resulting URL.
+    return urlappend(subdir_url, filename)

